@@ -8,6 +8,8 @@ from urllib import quote
 import socket
 import threading
 
+from zope.cachedescriptors.property import Lazy
+
 from AccessControl.SecurityManagement import noSecurityManager
 from Acquisition.interfaces import IAcquirer
 from ZPublisher.BaseRequest import exec_callables, RequestContainer
@@ -19,7 +21,7 @@ from zope.component import queryMultiAdapter, getMultiAdapter
 from zope.event import notify
 from zope.interface import implements
 from zope.site.hooks import getSite
-from zope.publisher.interfaces.browser import IBrowserPage
+from zope.publisher.interfaces.browser import IBrowserPage, IBrowserPublisher
 from zope.security.management import newInteraction, endInteraction
 import Zope2
 import zExceptions
@@ -73,11 +75,15 @@ class WSGIResult(object):
     """
 
     def __init__(self, request, publisher, data, cleanup=None):
-        self.__next = iter(data).next
         self.request = request
         self.publisher = publisher
         self.cleanup = cleanup
-        self.__iteration_error = False
+        self.__next = iter(data).next
+        self.__failed = False
+
+    @Lazy
+    def url(self):
+        return reconstruct_url_from_environ(self.request.environ)
 
     def next(self):
         try:
@@ -85,7 +91,9 @@ class WSGIResult(object):
         except StopIteration:
             raise
         except Exception:
-            self.__iteration_error = True
+            logger.exception(
+                u"Unexpected error in the WSGI stack for '%s'", self.url)
+            self.__failed = True
             raise
 
     def __iter__(self):
@@ -93,11 +101,10 @@ class WSGIResult(object):
 
     def close(self):
         try:
-            if self.__iteration_error:
+            if self.__failed:
                 logger.error(
                     "An error happened in the WSGI stack "
-                    "while iterating the result for the url '%s'" %
-                    reconstruct_url_from_environ(self.request.environ))
+                    "while iterating the result for the url '%s'", self.url)
                 self.publisher.abort()
             else:
                 self.publisher.finish()
@@ -105,8 +112,8 @@ class WSGIResult(object):
             self.publisher.abort()
             raise
         finally:
+            # Always cleanup
             if self.cleanup is not None:
-                # Always cleanup
                 self.cleanup()
 
 
@@ -205,17 +212,25 @@ class WSGIPublication(object):
         context = DefaultError(error)
         if IAcquirer.providedBy(last_known_obj):
             context = context.__of__(last_known_obj)
-        error_page = queryMultiAdapter(
+        error_view = queryMultiAdapter(
             (context, self.request), name='error.html')
+        if IBrowserPublisher.providedBy(error_view):
+            error_view, error_path = error_view.browserDefault(self.request)
+            if error_path:
+                raise NotImplementedError(
+                    u'Error browserDefault retuned an path. '
+                    u'This is not implemented.')
 
-        if error_page is not None:
+        if error_view is not None:
+            notify(interfaces.PublicationBeforeError(
+                    self.request, last_known_obj))
             try:
-                error_result = error_page()
+                error_result = error_view()
                 if error_result is not None:
                     self.response.setBody(error_result)
 
                 notify(interfaces.PublicationAfterRender(
-                        self.request, error_page))
+                        self.request, error_view))
 
             except Exception as error:
                 log_last_error(
